import { Children, cloneElement, useEffect, useState, useRef, forwardRef } from 'react';
import PropTypes from 'prop-types';
import useControlledState from '@s-ui/react-hooks/lib/useControlledState';
import loadable from '@loadable/component';
import { PLACEMENTS, TRIGGERS } from './config';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
import { Fragment as _Fragment } from "react/jsx-runtime";
var BASE_CLASS = 'sui-AtomPopover';
var CLASS_INNER = BASE_CLASS + "-inner";
var PREFIX_PLACEMENT = BASE_CLASS + "-";
var DEFAULT_OFFSET = 'auto,4px';
var DEFAULT_TRIGGER = TRIGGERS.LEGACY;
var DEFAULT_DELAY = 0; // https://github.com/reactstrap/reactstrap/blob/7.1.0/src/TooltipPopoverWrapper.js

var Popover = loadable(function () {
  return import('reactstrap/lib/Popover');
}, {
  ssr: true
});
var AtomPopover = /*#__PURE__*/forwardRef(function (_ref, outRef) {
  var children = _ref.children,
      closeIcon = _ref.closeIcon,
      content = _ref.content,
      _ref$onClose = _ref.onClose,
      onClose = _ref$onClose === void 0 ? function () {} : _ref$onClose,
      _ref$onOpen = _ref.onOpen,
      onOpen = _ref$onOpen === void 0 ? function () {} : _ref$onOpen,
      _ref$placement = _ref.placement,
      placement = _ref$placement === void 0 ? PLACEMENTS.BOTTOM : _ref$placement,
      isVisible = _ref.isVisible,
      defaultIsVisible = _ref.defaultIsVisible,
      _ref$hideArrow = _ref.hideArrow,
      hideArrow = _ref$hideArrow === void 0 ? true : _ref$hideArrow,
      _ref$trigger = _ref.trigger,
      trigger = _ref$trigger === void 0 ? DEFAULT_TRIGGER : _ref$trigger;
  var targetRef = useRef();

  var _useState = useState(),
      ready = _useState[0],
      setReady = _useState[1];

  var _useControlledState = useControlledState(isVisible, defaultIsVisible),
      isVisibleState = _useControlledState[0],
      setIsVisibleState = _useControlledState[1];

  useEffect(function () {
    setReady(targetRef.current);
  }, [targetRef, setReady]);

  var extendChildren = function extendChildren() {
    var onClick = function onClick(onClickHandler) {
      if (onClickHandler === void 0) {
        onClickHandler = function onClickHandler() {
          return null;
        };
      }

      return function (ev) {
        onOpen();
        setIsVisibleState(true);
        onClickHandler(ev);
      };
    };

    var childrenOnly = typeof children === 'string' ? [/*#__PURE__*/_jsx("span", {
      children: children
    }, 1)] : Children.only(children);
    var response = Children.map(childrenOnly, function (child, key) {
      var attrs = {
        onClick: onClick(child.props.onClick)
      };

      attrs.ref = function (node) {
        ;
        [child.ref, targetRef].forEach(function (ref) {
          if (typeof ref === 'function') {
            ref(node);
          } else if (ref !== null) {
            ref.current = node;
          }
        });
      };

      attrs.key = key;
      return /*#__PURE__*/cloneElement(child, attrs);
    });
    return response;
  };

  var handleToggle = function handleToggle(ev) {
    setIsVisibleState(!isVisibleState);
    onClose(ev);
  };

  return /*#__PURE__*/_jsxs(_Fragment, {
    children: [extendChildren(), isVisibleState && ready && /*#__PURE__*/_jsxs(Popover, {
      className: BASE_CLASS,
      delay: DEFAULT_DELAY,
      innerClassName: CLASS_INNER,
      hideArrow: hideArrow,
      arrowClassName: BASE_CLASS + "-arrow",
      isOpen: isVisibleState,
      offset: DEFAULT_OFFSET,
      placement: placement,
      placementPrefix: PREFIX_PLACEMENT,
      target: targetRef.current,
      toggle: handleToggle,
      trigger: Array.isArray(trigger) ? trigger.join(' ') : trigger,
      innerRef: outRef,
      children: [closeIcon && /*#__PURE__*/_jsx("div", {
        className: BASE_CLASS + "-closeIcon",
        onClick: handleToggle,
        children: closeIcon
      }), content]
    })]
  });
});
AtomPopover.displayName = 'AtomPopover';
export { PLACEMENTS as atomPopoverPositions };
export { TRIGGERS as atomPopoverTriggers };
export default AtomPopover;